import { syntaxTree } from "@codemirror/language";
const VAL_NODE_NAME = /^(?:Null|True|False|Object|Array|String|Number)$/;
// borrowed from `codemirror-json5`
// TODO: determine from spec if {"prop'name": example} is valid in json5
// JSON5 parse doesn't support parsing individual strings like JSON.parse does
function json5PropNameParser(s) {
    if (s.length < 2)
        return s;
    let first = s[0];
    let last = s[s.length - 1];
    if ((first === `'` && last === `'`) || (first === `"` && last === `"`)) {
        s = s.slice(1, -1);
    }
    return s;
}
const propNameParsers = {
    json4: JSON.parse,
    json5: json5PropNameParser,
};
/**
 * get a JSON4/5 pointer for a given node in the editor
 *
 * adapted from https://discuss.codemirror.net/t/json-pointer-at-cursor-seeking-implementation-critique/4793/3
 * this could be useful for other things later!
 * @group Utilities
 * @internal
 */
export function getJsonPointerAt(docText, node, mode = "json4") {
    const path = [];
    // retrieve a simple parser for parsing the property name
    const parse = propNameParsers[mode];
    for (let n = node; n && n.parent; n = n.parent) {
        switch (n.parent.name) {
            case "Property": {
                const name = n.parent.getChild("PropertyName");
                if (name) {
                    path.unshift(parse(docText.sliceString(name.from, name.to)).replace(/[\/~]/g, (v) => (v === "~" ? "~0" : "~1")));
                }
                break;
            }
            case "Array": {
                if (VAL_NODE_NAME.test(n.name)) {
                    let index = 0;
                    for (let s = n.prevSibling; s; s = s.prevSibling) {
                        if (VAL_NODE_NAME.test(s.name)) {
                            index++;
                        }
                    }
                    path.unshift("" + index);
                }
                break;
            }
        }
    }
    path.unshift("");
    return path.join("/");
}
/**
 * retrieve a JSON pointer for a given position in the editor
 * @group Utilities
 */
export const jsonPointerForPosition = (state, pos, side, mode = "json4") => {
    return getJsonPointerAt(state.doc, syntaxTree(state).resolve(pos, side), mode);
};
/**
 * retrieve a Map of all the json pointers in a document
 * @group Utilities
 */
export const getJsonPointers = (state, mode = "json4") => {
    const json = syntaxTree(state);
    const pointers = new Map();
    json.iterate({
        enter: (type) => {
            var _a, _b, _c, _d, _e, _f, _g, _h;
            if (type.name === "PropertyName") {
                const pointer = getJsonPointerAt(state.doc, type.node, mode);
                const { from: keyFrom, to: keyTo } = type.node;
                // if there's no value, we can't get the valueFrom/to
                if (!((_b = (_a = type.node) === null || _a === void 0 ? void 0 : _a.nextSibling) === null || _b === void 0 ? void 0 : _b.node)) {
                    pointers.set(pointer, { keyFrom, keyTo });
                    return true;
                }
                const nextNode = mode === "json4"
                    ? (_d = (_c = type.node) === null || _c === void 0 ? void 0 : _c.nextSibling) === null || _d === void 0 ? void 0 : _d.node
                    : (_h = (_g = (_f = (_e = type.node) === null || _e === void 0 ? void 0 : _e.nextSibling) === null || _f === void 0 ? void 0 : _f.node) === null || _g === void 0 ? void 0 : _g.nextSibling) === null || _h === void 0 ? void 0 : _h.node;
                if (!nextNode) {
                    pointers.set(pointer, { keyFrom, keyTo });
                    return true;
                }
                const { from: valueFrom, to: valueTo } = nextNode;
                pointers.set(pointer, { keyFrom, keyTo, valueFrom, valueTo });
                return true;
            }
        },
    });
    return pointers;
};
